/*
 * This code is based on an example provided by Richard Stanford, 
 * a tutorial reader.
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

public class DynamicTree extends JPanel 
{
    protected DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

    public DynamicTree() 
    {
    	BuddyStatus buddyStatus = new BuddyStatus("Buddy", "Buddy", "0", "0");
        rootNode = new DefaultMutableTreeNode(buddyStatus);
        treeModel = new DefaultTreeModel(rootNode);
        treeModel.addTreeModelListener(new MyTreeModelListener());
		
		tree = new JTree(treeModel);
        tree.setEditable(true);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
        tree.setCellRenderer(new BuddyRenderer());

        JScrollPane scrollPane = new JScrollPane(tree);
        setLayout(new GridLayout(1,0));
        add(scrollPane);
    }

    /** Remove all nodes except the root node. */
    public void clear() 
    {
        rootNode.removeAllChildren();
        treeModel.reload();
    }

    /** Remove the currently selected node. */
    public void removeCurrentNode() 
    {
        TreePath currentSelection = tree.getSelectionPath();
        if (currentSelection != null) 
        {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
            MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
            if (parent != null) 
            {
                treeModel.removeNodeFromParent(currentNode);
                return;
            }
        } 
        toolkit.beep();
    }

    /** Add child to the currently selected node. */
    public DefaultMutableTreeNode addObject(Object child) 
    {
        DefaultMutableTreeNode parentNode = null;
        TreePath parentPath = tree.getSelectionPath();
        if (parentPath == null) 
        {
            parentNode = rootNode;
        } 
        else 
        {
            parentNode = (DefaultMutableTreeNode)(parentPath.getLastPathComponent());
        }
        return addObject(parentNode, child, true);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Object child) 
    {
        return addObject(parent, child, false);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,Object child, boolean shouldBeVisible) 
    {
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
        if (parent == null) 
        {
            parent = rootNode;
        }

        treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

        // Make sure the user can see the lovely new node.
        if (shouldBeVisible) 
        {
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        }
        return childNode;
    }
    

	public String getBuddy()
	{
        TreePath currentSelection = tree.getSelectionPath();
		
        if (currentSelection != null) 
        {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
            MutableTreeNode parentNode = (MutableTreeNode)(currentNode.getParent());
			if(currentNode.isLeaf())
			{
				System.out.println(" currnent nod is leaf");
				return currentNode.toString();
			}
			else
			{
				System.out.println(" currnent node is folder, node name : " + currentNode.toString());
				return currentNode.toString();
			}
		}
		return null;
	}

	public String getBuddyGroup()
	{
        TreePath currentSelection = tree.getSelectionPath();
		
        if (currentSelection != null) 
        {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
            MutableTreeNode parentNode = (MutableTreeNode)(currentNode.getParent());
			if(currentNode.isLeaf() && parentNode != null)
			{
				System.out.println(" currnent nod is leaf, parent node name : " + currentNode.getParent().toString());
				return parentNode.toString();
			}
			else
			{
				System.out.println(" currnent node is folder, node name : " + currentNode.toString());
				return currentNode.toString();
			}
		}
		return null;
	}

    class MyTreeModelListener implements TreeModelListener 
    {
        public void treeNodesChanged(TreeModelEvent e) 
        {
            DefaultMutableTreeNode node;
            node = (DefaultMutableTreeNode)(e.getTreePath().getLastPathComponent());

            /*
             * If the event lists children, then the changed
             * node is the child of the node we've already
             * gotten.  Otherwise, the changed node and the
             * specified node are the same.
             */
            try {
                int index = e.getChildIndices()[0];
                node = (DefaultMutableTreeNode)(node.getChildAt(index));
            } catch (NullPointerException exc) {}

            System.out.println("The user has finished editing the node.");
            System.out.println("New value: " + node.getUserObject());
        }
        public void treeNodesInserted(TreeModelEvent e) 
        {
	       	p.r(this,"treeNodesInserted",e.getTreePath().getLastPathComponent());
        }
        public void treeNodesRemoved(TreeModelEvent e) 
        {
        	p.r(this,"treeNodesRemoved",e.getTreePath().getLastPathComponent());
        }
        public void treeStructureChanged(TreeModelEvent e) 
        {
        	p.r(this,"treeStructureChanged",e.getTreePath().getLastPathComponent());
        }
    }
}
